function out = colonyobserver(traj)
% Function COLONYOBSERVER runs the forward and backward observers on colony
% data generated by the script SSACOLONY. It was used to generate Figures
% 3(c) and 3(d).

% rates from Hayot, Science Signalling, 2011
k = [1e-2 5.8e-3 6e-3 1.92e-4];

% initialize probability distribution
p0 = zeros(656,1);
p0(s2i(10,0)) = 1;

% set up stochastic matrix
for m = 0:15
    for p = 0:40
        %Reaction 1
        if m ~= 15
            Q(s2i(m,p),s2i(m+1,p)) = k(1);
        end
        %Reaction 2
        if p ~= 40
            Q(s2i(m,p),s2i(m,p+1)) = m*k(2);
        end
        %Reaction 3
        if m ~= 0
            Q(s2i(m,p),s2i(m-1,p)) = m*k(3);
        end
        %Reaction 4
        if p ~= 0
            Q(s2i(m,p),s2i(m,p-1)) = p*k(4);
        end
    end;
end;

Q = Q' - diag(sum(Q'));

% pre-calculate some useful time step matrices
Q1 = expm(Q*60);
Q20 = expm(Q*1200);

p20 = Q20*p0;
p40 = Q20*split_cells(p20);
p60 = Q20*split_cells(p40);

% initialize forward observer
for c = 1:8
    PF{c} = [];
end;

% for each of the fourth-generation cells, run the forward observer
for c = [1 3 6 8]
    Pr = Q1*p60; PF{c} = [p60 Pr];
    keep = (0:41:615) + traj{c}.Pobs(1) + 1;
    Pr(setdiff(1:656,keep)) = 0; Pr = Pr/sum(Pr);
    out{c}(:,1) = Pr(keep); PF2{c} = [p60 Pr];
    
    for i=2:20
        Pr = Q1 * Pr; PF{c} = [PF{c} Pr];
        keep = (0:41:615) + traj{c}.Pobs(i) + 1;
        Pr(setdiff(1:656,keep)) = 0; Pr = Pr/sum(Pr);
        out{c}(:,i) = Pr(keep); PF2{c} = [PF2{c} Pr];
    end;
end

% for each of the third-generation cells, run the forward observer
for c = [2 7]
    Pr = Q1*p40; PF{c} = [p40 Pr];
    keep = (0:41:615) + traj{c}.Pobs(1) + 1;
    Pr(setdiff(1:656,keep)) = 0; Pr = Pr/sum(Pr);
    out{c}(:,1) = Pr(keep); PF2{c} = [p40 Pr];
    
    for i=2:40
        Pr = Q1 * Pr; PF{c} = [PF{c} Pr];
        keep = (0:41:615) + traj{c}.Pobs(i) + 1;
        Pr(setdiff(1:656,keep)) = 0; Pr = Pr/sum(Pr);
        out{c}(:,i) = Pr(keep);
        if i == 20
            out{c}(:,i) = divide_cell(out{c}(:,i));
            Pr(keep) = out{c}(:,i);
        end;
        PF2{c} = [PF2{c} Pr];
    end;
end

% for the second-generation cell, run the forward observer
for c = 5
    Pr = Q1*p20; PF{c} = [p20 Pr];
    keep = (0:41:615) + traj{c}.Pobs(1) + 1;
    Pr(setdiff(1:656,keep)) = 0; Pr = Pr/sum(Pr);
    out{c}(:,1) = Pr(keep); PF2{c} = [p20 Pr];
    
    for i=2:60
        Pr = Q1 * Pr; PF{c} = [PF{c} Pr];
        keep = (0:41:615) + traj{c}.Pobs(i) + 1;
        Pr(setdiff(1:656,keep)) = 0; Pr = Pr/sum(Pr);
        out{c}(:,i) = Pr(keep);
        if ismember(i,[20 40])
            out{c}(:,i) = divide_cell(out{c}(:,i));
            Pr(keep) = out{c}(:,i);
        end;
        PF2{c} = [PF2{c} Pr];
    end;
end

% for the first-generation cell, run the forward observer
for c = 4
    Pr = Q1*p0; PF{c} = [p0 Pr];
    keep = (0:41:615) + traj{c}.Pobs(1) + 1;
    Pr(setdiff(1:656,keep)) = 0; Pr = Pr/sum(Pr);
    out{c}(:,1) = Pr(keep); PF2{c} = [p0 Pr];
    
    for i=2:80
        Pr = Q1 * Pr; PF{c} = [PF{c} Pr];
        keep = (0:41:615) + traj{c}.Pobs(i) + 1;
        Pr(setdiff(1:656,keep)) = 0; Pr = Pr/sum(Pr);
        out{c}(:,i) = Pr(keep);
        if ismember(i,[20 40 60])
            out{c}(:,i) = divide_cell(out{c}(:,i));
            Pr(keep) = out{c}(:,i);
        end;
        PF2{c} = [PF2{c} Pr];
    end;
end

% initialize the backward observer
for c = 1:8
    PB{c} = PF2{c}(:,end);
    out2{c} = PB{c}(PB{c} > 0);
end;

% for all of the cells, run the backward observer for the fourth generation
for i=1:19
    f3 = Q1;
    for c=1:8
        f1 = PB{c}(:,1);
        f2 = PF{c}(:,end-i+1);
        f4 = PF2{c}(:,end-i);
        foo = (f1' ./ f2') * f3 * diag(f4); foo = foo ./ sum(foo);
        PB{c} = [foo' PB{c}];
        out2{c} = [foo(foo>0)' out2{c}];
    end;
end;

% Pinch the youngest set of cells. (1-2, 3-4, 5-6, 7-8)
z1 = 1./p60;
for c = [ 1 3 5 7 ]
    bar = zeros(656,1);
    z2 = Q1 * PB{c}(:,1);
    z3 = Q1 * PB{c+1}(:,1);
    bar = z2 .* z3; bar = bar/sum(bar);
    bar = z1 .* bar; bar = bar/sum(bar);
    bar = pinch_cells(bar);
    if ismember(c,[1 3])
        f1 = bar;
        f2 = PF{c+1}(:,end-19);
        f4 = PF2{c+1}(:,end-20);
        foo = (f1' ./ f2') * f3 * diag(f4); foo = foo ./ sum(foo);
        PB{c+1} = [foo' PB{c+1}];
        out2{c+1} = [foo(foo>0)' out2{c+1}];
    else
        f1 = bar;
        f2 = PF{c}(:,end-19);
        f4 = PF2{c}(:,end-20);
        foo = (f1' ./ f2') * f3 * diag(f4); foo = foo ./ sum(foo);
        PB{c} = [foo' PB{c}];
        out2{c} = [foo(foo>0)' out2{c}];
    end;
end;

% Run the backward observer for the third generation
for i=21:39
    f3 = Q1;
    for c=[2 4 5 7]
        f1 = PB{c}(:,1);
        f2 = PF{c}(:,end-i+1);
        f4 = PF2{c}(:,end-i);
        foo = (f1' ./ f2') * f3 * diag(f4); foo = foo ./ sum(foo);
        PB{c} = [foo' PB{c}];
        out2{c} = [foo(foo>0)' out2{c}];
    end;
end;

% Pinch the second set of cells. (2-4,5-7)
z1 = 1./p40;
for c = [ 2 5 ]
    bar = zeros(656,1);
    z2 = Q1 * PB{c}(:,1);
    z3 = Q1 * PB{c+2}(:,1);
    bar = z2 .* z3; bar = bar/sum(bar);
    bar = z1 .* bar; bar = bar/sum(bar);
    bar = pinch_cells(bar);
    if c == 2
        f1 = bar;
        f2 = PF{c+2}(:,end-39);
        f4 = PF2{c+2}(:,end-40);
        foo = (f1' ./ f2') * f3 * diag(f4); foo = foo ./ sum(foo);
        PB{c+2} = [foo' PB{c+2}];
        out2{c+2} = [foo(foo>0)' out2{c+2}];
    else
        f1 = bar;
        f2 = PF{c}(:,end-39);
        f4 = PF2{c}(:,end-40);
        foo = (f1' ./ f2') * f3 * diag(f4); foo = foo ./ sum(foo);
        PB{c} = [foo' PB{c}];
        out2{c} = [foo(foo>0)' out2{c}];
    end;
end;

% Run the backward observer for the second generation
for i=41:59
    f3 = Q1;
    for c=[ 4 5 ]
        f1 = PB{c}(:,1);
        f2 = PF{c}(:,end-i+1);
        f4 = PF2{c}(:,end-i);
        foo = (f1' ./ f2') * f3 * diag(f4); foo = foo ./ sum(foo);
        PB{c} = [foo' PB{c}];
        out2{c} = [foo(foo>0)' out2{c}];
    end;
end;

% Pinch the third set of cells. (4-5)
z1 = 1./p20;
for c = 4
    bar = zeros(656,1);
    z2 = Q1 * PB{c}(:,1);
    z3 = Q1 * PB{c+1}(:,1);
    bar = z2 .* z3; bar = bar/sum(bar);
    bar = z1 .* bar; bar = bar/sum(bar);
    bar = pinch_cells(bar);
        f1 = bar;
        f2 = PF{c}(:,end-59);
        f4 = PF2{c}(:,end-60);
        foo = (f1' ./ f2') * f3 * diag(f4); foo = foo ./ sum(foo);
        PB{c} = [foo' PB{c}];
        out2{c} = [foo(foo>0)' out2{c}];
end;

% Run the backward observer for the ancestral cell
for i=61:79
    f3 = Q1;
    for c= 4
        f1 = PB{c}(:,1);
        f2 = PF{c}(:,end-i+1);
        f4 = PF2{c}(:,end-i);
        foo = (f1' ./ f2') * f3 * diag(f4); foo = foo ./ sum(foo);
        PB{c} = [foo' PB{c}];
        out2{c} = [foo(foo>0)' out2{c}];
    end;
end;

% plot the outputs
cmap = [zeros(255,1) linspace(0,1,255)' zeros(255,1)];

figure
for c=1:8
    subplot(8,1,c)
    if ismember(c, [1 3 6 8])
        out{c} = [zeros(16,60) out{c}];
    elseif ismember(c, [2 7])
        out{c} = [zeros(16,40) out{c}];
    elseif c == 5
        out{c} = [zeros(16,20) out{c}];
    end;
    imagesc(out{c},[0 .6])
    colormap(cmap)
    axis xy
    ylim([.5 11.5])
    set(gca,'YTick',[1 6 11])
    set(gca,'YTickLabel',{'0' '5' '10'})
    if c ~= 8
        set(gca,'XTick',[])
    end;
end

figure
for c=1:8
    subplot(8,1,c)
    if ismember(c, [1 3 6 8])
        out2{c} = [zeros(16,60) out2{c}];
    elseif ismember(c, [2 7])
        out2{c} = [zeros(16,40) out2{c}];
    elseif c == 5
        out2{c} = [zeros(16,20) out2{c}];
    end;
    imagesc(out2{c},[0 .6])
    colormap(cmap)
    axis xy
    ylim([.5 11.5])
    set(gca,'YTick',[1 6 11])
    set(gca,'YTickLabel',{'0' '5' '10'})
    if c ~= 8
        set(gca,'XTick',[])
    end;
end

end

 function pnew = divide_cell(p)
 pnew = zeros(16,1);
 for m = 0:15
    for i=0:m
            pnew(i+1) = pnew(i+1) + binopdf(i,m,.5)*p(m+1);
    end
end;
 end
 
function out = s2i(M,P)
 out = 41*M + P + 1;
end

function [M,P] = i2s(ix)
ix = ix - 1;
M = fix(ix/41);
P = rem(ix,41);
end

